package net.sf.xanswer.indexing;

/**
 * This is an interface that is to be implemented by all index structures.
 *
 */
public interface IndexApi
{
    /**
     * Creates an index, using specified location. If the specified
     * location does not exist, it should be created.
     *
     * @param location String representation of absoulute path
     * to your location (may be a file or a directory as well).
     *
     * @throws IndexException if database creation failed for some reasons
     */
    public void create(String location) throws IndexException;


    /**
     * Opens previously created index.
     *
     * @throws IndexException if an error accured while opening the index.
     */
    public void open() throws IndexException;


    /**
     * Closes the index, flushes all caches.
     *
     * @throws IndexException if an error accured while closing the index.
     */
    public void close() throws IndexException;

}
